/*
 * arch/arm/mach-ep93xx/snapper_cl15.c
 * Bluewater Systems Snapper CL15 system module
 *
 * Copyright (C) 2009 Ryan Mallon <ryan@bluewatersys.com>
 * Copyright (C) 2009 Bluewater Systems Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 */

#include <linux/platform_device.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/i2c.h>
#include <linux/spi/spi.h>
#include <linux/spi/flash.h>
#include <linux/mtd/partitions.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <mach/spi.h>

#include <mach/ep93xx-regs.h>
#include <mach/platform.h>
#include <mach/fb.h>

/* FIXME - move to core.c */
static struct platform_device ep93xx_i2s_device = {
	.name		= "ep93xx-i2s",
	.id		= -1,
	/* FIXME - memory/irq resources */
};

static struct ep93xx_eth_data snapper_cl15_eth_data = {
	.phy_id		= 1,
};

static struct i2c_board_info __initdata snapper_cl15_i2c_data[] = {
	{
		/* Audio codec */
		I2C_BOARD_INFO("tlv320aic23", 0x1a),
	},
};

static void snapper_cl15_spi_flash_cs(int active, int active_high)
{
	/*
	 * We need to disable interrupts when accessing the SPI boot flash
	 * because otherwise the SPI fifos may drain and cause the chip
	 * select line to be de-asserted.
	 */
	if (active)
		local_irq_disable();
	else
		local_irq_enable();
}

static struct ep93xx_spi_chip snapper_cl15_spi_flash_controller = {
	.cs_control	= snapper_cl15_spi_flash_cs,
};

static struct mtd_partition snapper_cl15_spi_flash_partitions[] = {
	{
		.name		= "Bootloader",
		.offset		= 0,
		.size		= 0x40000,
	},
	{
		.name		= "Environment",
		.offset		= MTDPART_OFS_APPEND,
		.size		= MTDPART_SIZ_FULL,
	},
};
		
static struct flash_platform_data snapper_cl15_spi_flash_data = {
	.name		= "SPI Boot Flash",
	.parts		= snapper_cl15_spi_flash_partitions,
	.nr_parts	= ARRAY_SIZE(snapper_cl15_spi_flash_partitions),
};

static struct spi_board_info __initdata snapper_cl15_spi_data[] = {
	{
		/* SPI Flash */
		.modalias		= "sst25l",
		.max_speed_hz		= 3700000,
		.bus_num		= 0,
		.chip_select		= 0,
		.mode			= SPI_MODE_3,
		.platform_data		= &snapper_cl15_spi_flash_data,
		.controller_data	= &snapper_cl15_spi_flash_controller,
	},
};

static struct ep93xxfb_mach_info snapper_cl15_fb_info = {
	.num_modes	= EP93XXFB_USE_MODEDB,
	.bpp		= 16,
};

static void __init snapper_cl15_init_machine(void)
{
	ep93xx_init_devices();
	ep93xx_register_eth(&snapper_cl15_eth_data, 1);
	ep93xx_register_i2c(snapper_cl15_i2c_data, 
			    ARRAY_SIZE(snapper_cl15_i2c_data));
	ep93xx_register_spi(snapper_cl15_spi_data,
			    ARRAY_SIZE(snapper_cl15_spi_data));
	ep93xx_register_pcmcia();
	ep93xx_register_fb(&snapper_cl15_fb_info);

	platform_device_register(&ep93xx_i2s_device);
}

MACHINE_START(SNAPPER_CL15, "Bluewater Systems Snapper CL15")
	/* Maintainer: Ryan Mallon <ryan@bluewatersys.com> */
	.phys_io	= EP93XX_APB_PHYS_BASE,
	.io_pg_offst	= ((EP93XX_APB_VIRT_BASE) >> 18) & 0xfffc,
	.boot_params	= EP93XX_SDCE0_PHYS_BASE + 0x100,
	.map_io		= ep93xx_map_io,
	.init_irq	= ep93xx_init_irq,
	.timer 		= &ep93xx_timer,
	.init_machine	= snapper_cl15_init_machine,
MACHINE_END
