/*
 * drivers/pcmcia/ep93xx_pcmcia.h
 *
 * Author:	Ryan Mallon <ryan@bluewatersys.com>
 * Copyright:	Bluewater Systems
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef _EP93XX_PCMCIA_H
#define _EP93XX_PCMCIA_H

#include "soc_common.h"

#define EP93XX_PCATTRIB			0x20
#define EP93XX_PCCOMMON			0x24
#define EP93XX_PCIO			0x28
#define EP93XX_PCCTRL			0x40

#define EP93XX_PCCTRL_MODE_CF		0x01
#define EP93XX_PCCTRL_MODE_PCMCIA	0x02
#define EP93XX_PCCTRL_RESET		0x04
#define EP93XX_PCCTRL_WAIT_ENABLE	0x10

#define EP93XX_PCMCIA_WRPROT		0x01
#define EP93XX_PCMCIA_DETECT1		0x02
#define EP93XX_PCMCIA_DETECT2		0x04
#define EP93XX_PCMCIA_DETECT		0x06
#define EP93XX_PCMCIA_BVD1		0x08
#define EP93XX_PCMCIA_BVD2		0x10
#define EP93XX_PCMCIA_VS_3V		0x20
#define EP93XX_PCMCIA_READY		0x40
#define EP93XX_PCMCIA_VS_XV		0x80

#define EP93XX_PCMCIA_16BITS		0x80000000

typedef int (*ep93xx_pcmcia_probe_func_t)(struct device *dev);

/* Generic layer */
extern void ep93xx_pcmcia_write(unsigned int val, unsigned int reg);
extern unsigned int ep93xx_pcmcia_read(unsigned int reg);

extern void ep93xx_pcmcia_set_irqs(int *socket_irq, struct pcmcia_irqs *irqs, 
				   int nr_irqs);
extern int ep93xx_pcmcia_hw_init(struct soc_pcmcia_socket *skt);
extern void ep93xx_pcmcia_hw_shutdown(struct soc_pcmcia_socket *skt);
extern void ep93xx_pcmcia_socket_state(struct soc_pcmcia_socket *skt,
				       struct pcmcia_state *state);
extern int ep93xx_pcmcia_configure_socket(struct soc_pcmcia_socket *skt,
					  const socket_state_t *state);
extern void ep93xx_pcmcia_socket_init(struct soc_pcmcia_socket *skt);
extern void ep93xx_pcmcia_socket_suspend(struct soc_pcmcia_socket *skt);

extern int ep93xx_pcmcia_probe(ep93xx_pcmcia_probe_func_t probe_func,
			       struct pcmcia_low_level *ops);
extern void ep93xx_pcmcia_unregister(void);

#endif /* _EP93XX_PCMCIA_H */
